@@ -29,7 +29,19 @@ import locale
 import os
 import sys
 
+from gi import require_version as gi_require_version
+gi_require_version('GLib', '2.0')
+gi_require_version('Gio', '2.0')
+gi_require_version('Gtk', '3.0')
+gi_require_version('IBus', '1.0')
+
 from gi.repository import GLib
+from gi.repository import Gio
+
+# set_prgname before importing other modules to show the name in warning
+# messages when import modules are failed. E.g. Gtk.
+GLib.set_prgname('ibus-setup-libpinyin')
+
 from gi.repository import Gtk
 from gi.repository import IBus
 
@@ -37,42 +49,57 @@ import config
 from dicttreeview import DictionaryTreeView
 from shortcuteditor import ShortcutEditor
 
+DOMAINNAME = 'ibus-libpinyin'
 locale.setlocale(locale.LC_ALL, "")
 localedir = os.getenv("IBUS_LOCALEDIR")
 pkgdatadir = os.getenv("IBUS_PKGDATADIR") or "."
-gettext.install('ibus-libpinyin', localedir)
+
+# Python's locale module doesn't provide all methods on some
+# operating systems like FreeBSD
+try:
+    locale.bindtextdomain(DOMAINNAME, localedir)
+    locale.bind_textdomain_codeset(DOMAINNAME, 'UTF-8')
+except AttributeError:
+    pass
+
+gettext.bindtextdomain(DOMAINNAME, localedir)
+gettext.bind_textdomain_codeset(DOMAINNAME, 'UTF-8')
+
+gettext.install(DOMAINNAME, localedir)
 
 class PreferencesDialog:
     def __init__(self, engine):
         self.__bus = IBus.Bus()
-        self.__config = self.__bus.get_config()
         self.__builder = Gtk.Builder()
-        self.__builder.set_translation_domain("ibus-libpinyin")
+        self.__builder.set_translation_domain(DOMAINNAME)
         self.__builder.add_from_file("ibus-libpinyin-preferences.ui")
         self.__dialog = self.__builder.get_object("dialog")
         self.__init_pages()
         
-        if engine == "pinyin":
-            self.__config_namespace = "engine/Pinyin"
-            self.__values = dict(self.__config.get_values(self.__config_namespace))
+        if engine == "libpinyin":
+            self.__config_namespace = "com.github.libpinyin.ibus-libpinyin.libpinyin"
+            self.__config = Gio.Settings.new(self.__config_namespace)
             self.__init_general()
             self.__init_pinyin()
             self.__init_fuzzy()
             self.__init_dictionary()
             self.__init_user_data()
             self.__init_shortcut()
+            self.__init_cloud_input()
             self.__init_about()
-        elif engine == "bopomofo":
-            self.__config_namespace = "engine/Bopomofo"
-            self.__values = dict(self.__config.get_values(self.__config_namespace))
+        elif engine == "libbopomofo":
+            self.__config_namespace = "com.github.libpinyin.ibus-libpinyin.libbopomofo"
+            self.__config = Gio.Settings.new(self.__config_namespace)
             self.__init_general()
             self.__init_bopomofo()
             self.__init_fuzzy()
             self.__init_dictionary()
             #self.__init_user_data()
             self.__init_shortcut()
+            self.__init_bopomofo_cloud_input()
             self.__init_about()
             self.__convert_fuzzy_pinyin_to_bopomofo()
+            self.__display_style_box.hide()
 
         else:
             print("Error: Unknown Engine")
@@ -114,39 +141,58 @@ class PreferencesDialog:
         self.__init_trad = self.__builder.get_object("InitTraditionalChinese")
 
         # UI
-        self.__lookup_table_page_size = self.__builder.get_object("LookupTablePageSize")
+        self.__display_style = self.__builder.get_object("DisplayStyle")
+        self.__display_style_box = self.__builder.get_object("boxDisplayStyle")
         self.__lookup_table_orientation = self.__builder.get_object("LookupTableOrientation")
+        self.__lookup_table_page_size = self.__builder.get_object("LookupTablePageSize")
 
         self.__dynamic_adjust = self.__builder.get_object("DynamicAdjust")
         self.__remember_every_input = self.__builder.get_object("RememberEveryInput")
+        self.__show_suggestion = self.__builder.get_object("ShowSuggestion")
+        self.__emoji_candidate = self.__builder.get_object("EmojiCandidate")
+        self.__sort_candidate_option = self.__builder.get_object("SortCandidateOption")
 
         # read values
-        self.__init_chinese.set_active(self.__get_value("InitChinese", True))
-        self.__init_full.set_active(self.__get_value("InitFull", False))
-        self.__init_full_punct.set_active(self.__get_value("InitFullPunct", True))
-        self.__init_simp.set_active(self.__get_value("InitSimplifiedChinese", True))
+        self.__init_chinese.set_active(self.__get_value("init-chinese"))
+        self.__init_full.set_active(self.__get_value("init-full"))
+        self.__init_full_punct.set_active(self.__get_value("init-full-punct"))
+        self.__init_simp.set_active(self.__get_value("init-simplified-chinese"))
 
-        self.__lookup_table_orientation.set_active(self.__get_value("LookupTableOrientation", 0))
-        self.__lookup_table_page_size.set_value(self.__get_value("LookupTablePageSize", 5))
+        self.__lookup_table_orientation.set_active(self.__get_value("lookup-table-orientation"))
+        self.__lookup_table_page_size.set_value(self.__get_value("lookup-table-page-size"))
+        self.__display_style.set_active(self.__get_value("display-style"))
 
-        self.__dynamic_adjust.set_active(self.__get_value("DynamicAdjust", True))
-        self.__remember_every_input.set_active(self.__get_value("RememberEveryInput", False))
+        self.__dynamic_adjust.set_active(self.__get_value("dynamic-adjust"))
+        self.__remember_every_input.set_active(self.__get_value("remember-every-input"))
+        self.__show_suggestion.set_active(self.__get_value("show-suggestion"))
+        self.__emoji_candidate.set_active(self.__get_value("emoji-candidate"))
+        self.__sort_candidate_option.set_active(self.__get_value("sort-candidate-option"))
         # connect signals
-        self.__init_chinese.connect("toggled", self.__toggled_cb, "InitChinese")
-        self.__init_full.connect("toggled", self.__toggled_cb, "InitFull")
-        self.__init_full_punct.connect("toggled", self.__toggled_cb, "InitFullPunct")
-        self.__init_simp.connect("toggled", self.__toggled_cb, "InitSimplifiedChinese")
-        self.__dynamic_adjust.connect("toggled", self.__toggled_cb, "DynamicAdjust")
-        self.__remember_every_input.connect("toggled", self.__toggled_cb, "RememberEveryInput")
+        self.__init_chinese.connect("toggled", self.__toggled_cb, "init-chinese")
+        self.__init_full.connect("toggled", self.__toggled_cb, "init-full")
+        self.__init_full_punct.connect("toggled", self.__toggled_cb, "init-full-punct")
+        self.__init_simp.connect("toggled", self.__toggled_cb, "init-simplified-chinese")
+        self.__dynamic_adjust.connect("toggled", self.__toggled_cb, "dynamic-adjust")
+        self.__remember_every_input.connect("toggled", self.__toggled_cb, "remember-every-input")
+        self.__show_suggestion.connect("toggled", self.__toggled_cb, "show-suggestion")
+        self.__emoji_candidate.connect("toggled", self.__toggled_cb, "emoji-candidate")
 
-        def __lookup_table_page_size_changed_cb(adjustment):
-            self.__set_value("LookupTablePageSize", int(adjustment.get_value()))
+        def __display_size_changed_cb(widget):
+            self.__set_value("display-style", widget.get_active())
 
         def __lookup_table_orientation_changed_cb(widget):
-            self.__set_value("LookupTableOrientation", widget.get_active())
+            self.__set_value("lookup-table-orientation", widget.get_active())
 
+        def __lookup_table_page_size_changed_cb(adjustment):
+            self.__set_value("lookup-table-page-size", int(adjustment.get_value()))
+
+        def __sort_candidate_option_changed_cb(widget):
+            self.__set_value("sort-candidate-option", widget.get_active())
+
+        self.__display_style.connect("changed", __display_size_changed_cb)
         self.__lookup_table_orientation.connect("changed", __lookup_table_orientation_changed_cb)
         self.__lookup_table_page_size.connect("value-changed", __lookup_table_page_size_changed_cb)
+        self.__sort_candidate_option.connect("changed", __sort_candidate_option_changed_cb)
 
     def __init_pinyin(self):
         # page
@@ -159,12 +205,12 @@ class PreferencesDialog:
         self.__double_pinyin_schema = self.__builder.get_object("DoublePinyinSchema")
         # self.__double_pinyin_schema_label = self.__builder.get_object("labelDoublePinyinSchema")
         self.__double_pinyin_show_raw = self.__builder.get_object("DoublePinyinShowRaw")
-        self.__double_pinyin_show_raw.hide ()
+        self.__double_pinyin_show_raw.hide()
 
         # read value
-        self.__incomplete_pinyin.set_active(self.__get_value("IncompletePinyin", True))
-        self.__full_pinyin.set_active(not self.__get_value("DoublePinyin", False))
-        self.__double_pinyin_schema.set_active(self.__get_value("DoublePinyinSchema", 0))
+        self.__incomplete_pinyin.set_active(self.__get_value("incomplete-pinyin"))
+        self.__full_pinyin.set_active(not self.__get_value("double-pinyin"))
+        self.__double_pinyin_schema.set_active(self.__get_value("double-pinyin-schema"))
         if self.__full_pinyin.get_active():
             # self.__incomplete_pinyin.set_sensitive(True)
             self.__double_pinyin_schema.set_sensitive(False)
@@ -178,19 +224,19 @@ class PreferencesDialog:
 
         def __double_pinyin_toggled_cb(widget):
             val = widget.get_active()
-            self.__set_value("DoublePinyin", val)
+            self.__set_value("double-pinyin", val)
             self.__double_pinyin_schema.set_sensitive(val)
             # self.__double_pinyin_schema_label.set_sensitive(val)
             self.__double_pinyin_show_raw.set_sensitive(val)
 
         def __double_pinyin_schema_changed_cb(widget):
-            self.__set_value("DoublePinyinSchema", widget.get_active())
+            self.__set_value("double-pinyin-schema", widget.get_active())
 
         # connect signals
         self.__double_pinyin.connect("toggled", __double_pinyin_toggled_cb)
-        self.__incomplete_pinyin.connect("toggled", self.__toggled_cb, "IncompletePinyin")
+        self.__incomplete_pinyin.connect("toggled", self.__toggled_cb, "incomplete-pinyin")
         self.__double_pinyin_schema.connect("changed", __double_pinyin_schema_changed_cb)
-        self.__double_pinyin_show_raw.connect("toggled", self.__toggled_cb, "DoublePinyinShowRaw")
+        self.__double_pinyin_show_raw.connect("toggled", self.__toggled_cb, "double-pinyin-show-raw")
 
         self.__init_input_custom()
         self.__init_correct_pinyin()
@@ -213,27 +259,27 @@ class PreferencesDialog:
         self.__enter_key = self.__builder.get_object("CommitFirstCandidate")
 
         # read value
-        self.__bopomofo_keyboard_mapping.set_active(self.__get_value("BopomofoKeyboardMapping", 0))
-        self.__incomplete_bopomofo.set_active(self.__get_value("IncompletePinyin", False))
-        self.__select_keys.set_active(self.__get_value("SelectKeys", 0))
-        self.__guide_key.set_active(self.__get_value("GuideKey", 1))
-        self.__auxiliary_select_key_f.set_active(self.__get_value("AuxiliarySelectKey_F", 1))
-        self.__auxiliary_select_key_kp.set_active(self.__get_value("AuxiliarySelectKey_KP", 1))
-        self.__enter_key.set_active(self.__get_value("EnterKey", True))
+        self.__bopomofo_keyboard_mapping.set_active(self.__get_value("bopomofo-keyboard-mapping"))
+        self.__incomplete_bopomofo.set_active(self.__get_value("incomplete-pinyin"))
+        self.__select_keys.set_active(self.__get_value("select-keys"))
+        self.__guide_key.set_active(self.__get_value("guide-key"))
+        self.__auxiliary_select_key_f.set_active(self.__get_value("auxiliary-select-key-f"))
+        self.__auxiliary_select_key_kp.set_active(self.__get_value("auxiliary-select-key-kp"))
+        self.__enter_key.set_active(self.__get_value("enter-key"))
 
         # connect signals
         def __bopomofo_keyboard_mapping_changed_cb(widget):
-            self.__set_value("BopomofoKeyboardMapping", widget.get_active())
+            self.__set_value("bopomofo-keyboard-mapping", widget.get_active())
         def __select_keys_changed_cb(widget):
-            self.__set_value("SelectKeys", widget.get_active())
+            self.__set_value("select-keys", widget.get_active())
 
         self.__bopomofo_keyboard_mapping.connect("changed", __bopomofo_keyboard_mapping_changed_cb)
-        self.__incomplete_bopomofo.connect("toggled", self.__toggled_cb, "IncompletePinyin")
+        self.__incomplete_bopomofo.connect("toggled", self.__toggled_cb, "incomplete-pinyin")
         self.__select_keys.connect("changed", __select_keys_changed_cb)
-        self.__guide_key.connect("toggled", self.__toggled_cb, "GuideKey")
-        self.__auxiliary_select_key_f.connect("toggled", self.__toggled_cb, "AuxiliarySelectKey_F")
-        self.__auxiliary_select_key_kp.connect("toggled", self.__toggled_cb, "AuxiliarySelectKey_KP")
-        self.__enter_key.connect("toggled", self.__toggled_cb, "EnterKey")
+        self.__guide_key.connect("toggled", self.__toggled_cb, "guide-key")
+        self.__auxiliary_select_key_f.connect("toggled", self.__toggled_cb, "auxiliary-select-key-f")
+        self.__auxiliary_select_key_kp.connect("toggled", self.__toggled_cb, "auxiliary-select-key-kp")
+        self.__enter_key.connect("toggled", self.__toggled_cb, "enter-key")
 
     def __init_input_custom(self):
         # others
@@ -243,29 +289,29 @@ class PreferencesDialog:
         self.__auto_commit = self.__builder.get_object("AutoCommit")
 
         # read values
-        self.__shift_select_candidate.set_active(self.__get_value("ShiftSelectCandidate", False))
-        self.__minus_equal_page.set_active(self.__get_value("MinusEqualPage", True))
-        self.__comma_period_page.set_active(self.__get_value("CommaPeriodPage", True))
-        self.__auto_commit.set_active(self.__get_value("AutoCommit", False))
+        self.__shift_select_candidate.set_active(self.__get_value("shift-select-candidate"))
+        self.__minus_equal_page.set_active(self.__get_value("minus-equal-page"))
+        self.__comma_period_page.set_active(self.__get_value("comma-period-page"))
+        self.__auto_commit.set_active(self.__get_value("auto-commit"))
 
         # connect signals
-        self.__shift_select_candidate.connect("toggled", self.__toggled_cb, "ShiftSelectCandidate")
-        self.__minus_equal_page.connect("toggled", self.__toggled_cb, "MinusEqualPage")
-        self.__comma_period_page.connect("toggled", self.__toggled_cb, "CommaPeriodPage")
-        self.__auto_commit.connect("toggled", self.__toggled_cb, "AutoCommit")
+        self.__shift_select_candidate.connect("toggled", self.__toggled_cb, "shift-select-candidate")
+        self.__minus_equal_page.connect("toggled", self.__toggled_cb, "minus-equal-page")
+        self.__comma_period_page.connect("toggled", self.__toggled_cb, "comma-period-page")
+        self.__auto_commit.connect("toggled", self.__toggled_cb, "auto-commit")
 
     def __init_correct_pinyin(self):
         # auto correct
         self.__correct_pinyin = self.__builder.get_object("CorrectPinyin")
         self.__correct_pinyin_widgets = [
-            ("CorrectPinyin_GN_NG", True),
-            ("CorrectPinyin_MG_NG", True),
-            ("CorrectPinyin_IOU_IU", True),
-            ("CorrectPinyin_UEI_UI", True),
-            ("CorrectPinyin_UEN_UN", True),
-            ("CorrectPinyin_UE_VE", True),
-            ("CorrectPinyin_V_U", True),
-            ("CorrectPinyin_ON_ONG", True),
+            ('CorrectPinyin_GN_NG', 'correct-pinyin-gn-ng'),
+            ('CorrectPinyin_MG_NG', 'correct-pinyin-mg-ng'),
+            ('CorrectPinyin_IOU_IU', 'correct-pinyin-iou-iu'),
+            ('CorrectPinyin_UEI_UI', 'correct-pinyin-uei-ui'),
+            ('CorrectPinyin_UEN_UN', 'correct-pinyin-uen-un'),
+            ('CorrectPinyin_UE_VE', 'correct-pinyin-ue-ve'),
+            ('CorrectPinyin_V_U', 'correct-pinyin-v-u'),
+            ('CorrectPinyin_ON_ONG', 'correct-pinyin-on-ong'),
         ]
 
         def __correct_pinyin_toggled_cb(widget):
@@ -275,15 +321,15 @@ class PreferencesDialog:
         self.__correct_pinyin.connect("toggled", __correct_pinyin_toggled_cb)
 
         # init value
-        self.__correct_pinyin.set_active(self.__get_value("CorrectPinyin", True))
-        for name, defval in self.__correct_pinyin_widgets:
+        self.__correct_pinyin.set_active(self.__get_value("correct-pinyin"))
+        for name, keyname in self.__correct_pinyin_widgets:
             widget = self.__builder.get_object(name)
-            widget.set_active(self.__get_value(name, defval))
+            widget.set_active(self.__get_value(keyname))
 
-        self.__correct_pinyin.connect("toggled", self.__toggled_cb, "CorrectPinyin")
-        for name, defval in self.__correct_pinyin_widgets:
+        self.__correct_pinyin.connect("toggled", self.__toggled_cb, "correct-pinyin")
+        for name, keyname in self.__correct_pinyin_widgets:
             widget = self.__builder.get_object(name)
-            widget.connect("toggled", self.__toggled_cb, name)
+            widget.connect("toggled", self.__toggled_cb, keyname)
 
     def __init_fuzzy(self):
         # page Fuzzy
@@ -292,16 +338,16 @@ class PreferencesDialog:
         # fuzzy pinyin
         self.__fuzzy_pinyin = self.__builder.get_object("FuzzyPinyin")
         self.__fuzzy_pinyin_widgets = [
-            ("FuzzyPinyin_C_CH", True),
-            ("FuzzyPinyin_Z_ZH", True),
-            ("FuzzyPinyin_S_SH", True),
-            ("FuzzyPinyin_L_N", True),
-            ("FuzzyPinyin_F_H", True),
-            ("FuzzyPinyin_L_R", False),
-            ("FuzzyPinyin_G_K", False),
-            ("FuzzyPinyin_AN_ANG", True),
-            ("FuzzyPinyin_EN_ENG", True),
-            ("FuzzyPinyin_IN_ING", True),
+            ('FuzzyPinyin_C_CH', 'fuzzy-pinyin-c-ch'),
+            ('FuzzyPinyin_Z_ZH', 'fuzzy-pinyin-z-zh'),
+            ('FuzzyPinyin_S_SH', 'fuzzy-pinyin-s-sh'),
+            ('FuzzyPinyin_L_N', 'fuzzy-pinyin-l-n'),
+            ('FuzzyPinyin_F_H', 'fuzzy-pinyin-f-h'),
+            ('FuzzyPinyin_L_R', 'fuzzy-pinyin-l-r'),
+            ('FuzzyPinyin_G_K', 'fuzzy-pinyin-g-k'),
+            ('FuzzyPinyin_AN_ANG', 'fuzzy-pinyin-an-ang'),
+            ('FuzzyPinyin_EN_ENG', 'fuzzy-pinyin-en-eng'),
+            ('FuzzyPinyin_IN_ING', 'fuzzy-pinyin-in-ing'),
         ]
 
         def __fuzzy_pinyin_toggled_cb(widget):
@@ -311,15 +357,15 @@ class PreferencesDialog:
         self.__fuzzy_pinyin.connect("toggled", __fuzzy_pinyin_toggled_cb)
 
         # init value
-        self.__fuzzy_pinyin.set_active(self.__get_value("FuzzyPinyin", False))
-        for name, defval in self.__fuzzy_pinyin_widgets:
+        self.__fuzzy_pinyin.set_active(self.__get_value("fuzzy-pinyin"))
+        for name, keyname in self.__fuzzy_pinyin_widgets:
             widget = self.__builder.get_object(name)
-            widget.set_active(self.__get_value(name, defval))
+            widget.set_active(self.__get_value(keyname))
 
-        self.__fuzzy_pinyin.connect("toggled", self.__toggled_cb, "FuzzyPinyin")
-        for name, defval in self.__fuzzy_pinyin_widgets:
+        self.__fuzzy_pinyin.connect("toggled", self.__toggled_cb, "fuzzy-pinyin")
+        for name, keyname in self.__fuzzy_pinyin_widgets:
             widget = self.__builder.get_object(name)
-            widget.connect("toggled", self.__toggled_cb, name)
+            widget.connect("toggled", self.__toggled_cb, keyname)
 
     def __convert_fuzzy_pinyin_to_bopomofo(self):
         options = [
@@ -346,10 +392,10 @@ class PreferencesDialog:
         # dictionary tree view
         self.__dict_treeview = self.__builder.get_object("Dictionaries")
         self.__dict_treeview.show()
-        self.__dict_treeview.set_dictionaries(self.__get_value("Dictionaries", "2"))
+        self.__dict_treeview.set_dictionaries(self.__get_value("dictionaries"))
 
         def __notified_dicts_cb(self, param, dialog):
-            dialog.__set_value("Dictionaries", self.get_dictionaries())
+            dialog.__set_value("dictionaries", self.get_dictionaries())
 
         # connect notify signal
         self.__dict_treeview.connect("notify::dictionaries", __notified_dicts_cb, self)
@@ -389,10 +435,11 @@ class PreferencesDialog:
 
     def __import_dictionary_cb(self, widget):
         dialog = Gtk.FileChooserDialog \
-            (_("Please choose a file"), self.__dialog,
-             Gtk.FileChooserAction.OPEN,
-             (Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,
-              Gtk.STOCK_OPEN, Gtk.ResponseType.OK))
+            (title = _("Please choose a file"), parent = self.__dialog,
+             action = Gtk.FileChooserAction.OPEN)
+
+        dialog.add_buttons(Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,
+                           Gtk.STOCK_OPEN, Gtk.ResponseType.OK)
 
         filter_text = Gtk.FileFilter()
         filter_text.set_name("Text files")
@@ -401,16 +448,19 @@ class PreferencesDialog:
 
         response = dialog.run()
         if response == Gtk.ResponseType.OK:
-            self.__set_value("ImportDictionary", dialog.get_filename())
+            self.__set_value("import-dictionary", "")
+            self.__set_value("import-dictionary", dialog.get_filename())
 
         dialog.destroy()
 
     def __export_dictionary_cb(self, widget):
         dialog = Gtk.FileChooserDialog \
-                 (_("Please save a file"), self.__dialog,
-                  Gtk.FileChooserAction.SAVE,
-                  (Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,
-                   Gtk.STOCK_SAVE, Gtk.ResponseType.OK))
+                 (title = _("Please save a file"), parent = self.__dialog,
+                  action = Gtk.FileChooserAction.SAVE)
+
+        dialog.add_buttons(Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,
+                           Gtk.STOCK_SAVE, Gtk.ResponseType.OK)
+
         dialog.set_do_overwrite_confirmation(True)
 
         filter_text = Gtk.FileFilter()
@@ -420,12 +470,13 @@ class PreferencesDialog:
 
         response = dialog.run()
         if response == Gtk.ResponseType.OK:
-            self.__set_value("ExportDictionary", dialog.get_filename())
+            self.__set_value("export-dictionary", "")
+            self.__set_value("export-dictionary", dialog.get_filename())
 
         dialog.destroy()
 
     def __clear_user_data_cb(self, widget, name):
-        self.__set_value("ClearUserData", name)
+        self.__set_value("clear-user-data", name)
 
     def __init_shortcut(self):
         # page Shortcut
@@ -441,11 +492,78 @@ class PreferencesDialog:
         self.__shortcut_editor.connect("shortcut-changed", self.__shortcut_changed_cb)
 
         # set shortcuts
-        self.__shortcut_editor.update_shortcuts(self.__values)
+        self.__shortcut_editor.update_shortcuts(self.__config)
 
     def __shortcut_changed_cb(self, editor, key, value):
         self.__set_value(key, value)
 
+    def __init_bopomofo_cloud_input(self):
+        # init state
+        self.__frame_cloud_input = self.__builder.get_object("frameBopomofoCloudInput")
+        self.__init_enable_cloud_input = self.__builder.get_object("InitEnableBopomofoCloudInput")
+
+        # cloud input option
+        self.__cloud_input_source = self.__builder.get_object("BopomofoCloudInputSource")
+
+        if not config.support_cloud_input():
+            self.__frame_cloud_input.hide()
+
+        # read values
+        self.__init_enable_cloud_input.set_active(self.__get_value("enable-cloud-input"))
+
+        self.__cloud_input_source.set_active(self.__get_value("cloud-input-source"))
+
+        if self.__init_enable_cloud_input.get_active():
+            self.__cloud_input_source.set_sensitive(True)
+        else:
+            self.__cloud_input_source.set_sensitive(False)
+
+        # connect signals
+        def __enable_cloud_input_cb(widget):
+            val = widget.get_active()
+            self.__set_value("enable-cloud-input", val)
+            self.__cloud_input_source.set_sensitive(val)
+
+        def __cloud_input_source_changed_cb(widget):
+            self.__set_value("cloud-input-source", widget.get_active())
+
+        self.__init_enable_cloud_input.connect("toggled", __enable_cloud_input_cb)
+        self.__cloud_input_source.connect("changed", __cloud_input_source_changed_cb)
+
+
+    def __init_cloud_input(self):
+        # init state
+        self.__frame_cloud_input = self.__builder.get_object("frameCloudInput")
+        self.__init_enable_cloud_input = self.__builder.get_object("InitEnableCloudInput")
+
+        # cloud input option
+        self.__cloud_input_source = self.__builder.get_object("CloudInputSource")
+
+        if not config.support_cloud_input():
+            self.__frame_cloud_input.hide()
+
+        # read values
+        self.__init_enable_cloud_input.set_active(self.__get_value("enable-cloud-input"))
+
+        self.__cloud_input_source.set_active(self.__get_value("cloud-input-source"))
+
+        if self.__init_enable_cloud_input.get_active():
+            self.__cloud_input_source.set_sensitive(True)
+        else:
+            self.__cloud_input_source.set_sensitive(False)
+
+        # connect signals
+        def __enable_cloud_input_cb(widget):
+            val = widget.get_active()
+            self.__set_value("enable-cloud-input", val)
+            self.__cloud_input_source.set_sensitive(val)
+
+        def __cloud_input_source_changed_cb(widget):
+            self.__set_value("cloud-input-source", widget.get_active())
+
+        self.__init_enable_cloud_input.connect("toggled", __enable_cloud_input_cb)
+        self.__cloud_input_source.connect("changed", __cloud_input_source_changed_cb)
+
     def __init_about(self):
         # page About
         self.__page_about.show()
@@ -459,13 +577,18 @@ class PreferencesDialog:
     def __toggled_cb(self, widget, name):
         self.__set_value(name, widget.get_active ())
 
-    def __get_value(self, name, defval):
-        if name in self.__values:
-            var = self.__values[name]
-            if isinstance(defval, type(var)):
-                return var
-        self.__set_value(name, defval)
-        return defval
+    def __get_value(self, name):
+        var = self.__config.get_value(name)
+        vartype = var.get_type_string()
+        if vartype == 'b':
+            return var.get_boolean()
+        elif vartype == 'i':
+            return var.get_int32()
+        elif vartype == 's':
+            return var.get_string()
+        else:
+            print("var(%s) is not in support type." % repr(var), file=sys.stderr)
+            return None
 
     def __set_value(self, name, val):
         var = None
@@ -479,18 +602,17 @@ class PreferencesDialog:
             print("val(%s) is not in support type." % repr(val), file=sys.stderr)
             return
 
-        self.__values[name] = val
-        self.__config.set_value(self.__config_namespace, name, var)
+        self.__config.set_value(name, var)
 
     def run(self):
         return self.__dialog.run()
 
 def main():
-    name = "pinyin"
+    name = "libpinyin"
     if len(sys.argv) == 2:
         name = sys.argv[1]
-    if name not in ("pinyin", "bopomofo"):
-        name = "pinyin"
+    if name not in ("libpinyin", "libbopomofo"):
+        name = "libpinyin"
     PreferencesDialog(name).run()
 
 
